#ifndef _HATPLEXER_H
#define _HATPLEXER_H

#include<boost/regex.hpp>

using namespace std;

// ---------- HATP lib constants ----------
static const string REPORT_OK			= "OK";
static const string REPORT_FAIL			= "FAIL";
static const string REPORT_NOPLAN		= "NOPLAN";
static const string REPORT_ACHIEVED		= "ACHIEVED";
static const string LINK_CAUSAL			= "causal";
static const string LINK_HIERARCHICAL	= "hierarchical";
static const string TREE_TYPE			= "tree";
static const string STREAM_TYPE			= "stream";

// ---------- Default parsing ----------
static const string SPACE		= "[[:space:]]";
static const string WORD		= "([[:word:]]+)";
static const string LSCOPE		= "[{]";
static const string RSCOPE		= "[}]";
static const string LBRAC		= "[[.left-square-bracket.]]";
static const string RBRAC		= "[[.right-square-bracket.]]";
static const string JSEP		= "[[.colon.]]";
static const string COMMA		= "[[.comma.]]";
static const string QUOTE		= "[[.quotation-mark.]]";
static const string NUMBER		= "([[:digit:]]+)";
static const string FLOAT_NUMBER	= "([[:digit:]]+.[[:digit:]]+|[[:digit:]]+)";
static const string STRING		= QUOTE + WORD + QUOTE;
static const string SLIST		= LBRAC + "(((" + STRING + ")|(" + STRING + COMMA + "))*)" + RBRAC;
static const string NLIST		= LBRAC + "(((" + NUMBER + ")|(" + NUMBER + COMMA + "))*)" + RBRAC;

// ---------- HATP parsing ----------
static const string K_HATPREP	= QUOTE + "HATP-REP" + QUOTE;
static const string K_REQTYPE	= QUOTE + "REQ-TYPE" + QUOTE;
static const string K_TASKNAME	= QUOTE + "TASK-NAME" + QUOTE;
static const string K_REQID		= QUOTE + "REQ-ID" + QUOTE;
static const string K_REPORT	= QUOTE + "REPORT" + QUOTE;
static const string K_NODES		= QUOTE + "NODES" + QUOTE;
static const string K_STREAMS	= QUOTE + "STREAMS" + QUOTE;
static const string K_TREE		= QUOTE + "TREE" + QUOTE;
static const string K_ROOTS		= QUOTE + "ROOTS" + QUOTE;
static const string K_LLIST		= QUOTE + "LINKS" + QUOTE;
static const string K_NODE		= QUOTE + "NODE" + QUOTE;
static const string K_NODID		= QUOTE + "NODE-ID" + QUOTE;
static const string K_NODTYPE	= QUOTE + "NODE-TYPE" + QUOTE;
static const string K_ACTNAME	= QUOTE + "ACTION-NAME" + QUOTE;
static const string K_AGENTS	= QUOTE + "AGENTS" + QUOTE;
static const string K_STIME		= QUOTE + "START-TIME" + QUOTE;
static const string K_ETIME		= QUOTE + "END-TIME" + QUOTE;
static const string K_PARAMS	= QUOTE + "PARAMETERS" + QUOTE;
static const string K_LINK		= QUOTE + "LINK" + QUOTE;
static const string K_LTYPE		= QUOTE + "TYPE" + QUOTE;
static const string K_FROM		= QUOTE + "FROM" + QUOTE;
static const string K_TO		= QUOTE + "TO" + QUOTE;

static const string REQID		= K_REQID + JSEP + NUMBER;
static const string REQTYPE 	= K_REQTYPE + JSEP + STRING;
static const string TASKNAME	= K_TASKNAME + JSEP + STRING;
static const string REPORT		= K_REPORT + JSEP + STRING;
static const string NODID		= K_NODID + JSEP + NUMBER;
static const string NODTYPE		= K_NODTYPE + JSEP + STRING;
static const string ACTNAME		= K_ACTNAME + JSEP + STRING;
static const string AGENTS		= K_AGENTS + JSEP + SLIST;
static const string STTIME		= K_STIME + JSEP + FLOAT_NUMBER;
static const string ENTIME		= K_ETIME + JSEP + FLOAT_NUMBER;
static const string PARAMS		= K_PARAMS + JSEP + SLIST;
static const string LTYPE		= K_LTYPE + JSEP + STRING;
static const string LFROM		= K_FROM + JSEP + NUMBER;
static const string LTO			= K_TO + JSEP + NUMBER;

static const string NODE		= LSCOPE + K_NODE + JSEP + LSCOPE + NODID + COMMA + NODTYPE + COMMA + ACTNAME + COMMA + AGENTS + COMMA
								  + STTIME + COMMA + ENTIME + COMMA + PARAMS + RSCOPE + RSCOPE;
static const string LINK		= LSCOPE + K_LINK + JSEP + LSCOPE + LTYPE + COMMA + LFROM + COMMA + LTO + RSCOPE + RSCOPE;

static const string NODES		= K_NODES + JSEP + LBRAC+ "((("+ NODE +")|(" + NODE + COMMA + "))*)" + RBRAC;
static const string LINKS		= K_LLIST + JSEP + LBRAC + "((("+ LINK +")|(" + LINK + COMMA + "))*)" + RBRAC;
static const string ROOTS		= K_ROOTS + JSEP + NLIST;
static const string STREAMS		= K_STREAMS + JSEP + LSCOPE + ROOTS + COMMA + LINKS + RSCOPE;
static const string TREE		= K_TREE + JSEP + LSCOPE + ROOTS + COMMA + LINKS + RSCOPE;

static const string PTESTVAL	= LSCOPE + K_HATPREP + JSEP + LSCOPE + REQID + COMMA + K_REPORT + JSEP + QUOTE;
static const string PTESTOK		= LSCOPE + K_HATPREP + JSEP + LSCOPE + REQID + COMMA + K_REPORT + JSEP + QUOTE + REPORT_OK + QUOTE;
static const string PTESTNO		= LSCOPE + K_HATPREP + JSEP + LSCOPE + REQID + COMMA + K_REPORT + JSEP + QUOTE + REPORT_NOPLAN + QUOTE;
static const string PTESTAL		= LSCOPE + K_HATPREP + JSEP + LSCOPE + REQID + COMMA + K_REPORT + JSEP + QUOTE + REPORT_ACHIEVED + QUOTE;
static const string PLAN		= LSCOPE + K_HATPREP + JSEP + LSCOPE + REQID + COMMA + REPORT + COMMA + NODES + COMMA + STREAMS + COMMA + TREE + RSCOPE + RSCOPE;

// ---------- BOOST REGEX rules ----------
static const boost::regex regexPlan(PLAN);
static const boost::regex regexNode(NODE);
static const boost::regex regexLink(LINK);
static const boost::regex regexValidity(PTESTVAL);
static const boost::regex regexSNoPlan(PTESTNO);
static const boost::regex regexSAchieved(PTESTAL);

#endif
